/****************************************************************************/
// camdata.c
//
//  - populate & refresh container with photo data
//
/****************************************************************************/

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Cameraderie.
 *
 * The Initial Developer of the Original Code is Richard L. Walsh.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2006,2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

/****************************************************************************/

#include "camapp.h"
#include "camusb.h"

/****************************************************************************/

typedef struct _CAMGetData {
    ULONG       rc;
    BOOL        fSync;
    HWND        hReply;
    CAMPtr      thisCam;
    CAMObjectHandlePtr pHandles;
    CAMRecPtr   pcrFirst;
    CAMRecPtr   pcrLast;
    ULONG       cntRecords;
    ULONG       cntSync;
    CAMRecPtr * ppcrSync;
} CAMGetData;

/****************************************************************************/

void            GetData( HWND hwnd, BOOL fSync);
void            GetDataThreadProc( void * camgetdataptr);
void            GetDataReply( HWND hwnd, MPARAM mp1);
BOOL            GetList( HWND hCnr, CAMGetData * pcgd);
void            GetListThreadProc( void * camgetdataptr);
void            GetListReply( HWND hwnd, MPARAM mp1);
BOOL            SyncList( HWND hCnr, CAMGetData * pcgd);
void            SyncListThreadProc( void * camgetdataptr);
void            GetSyncReply( HWND hwnd, MPARAM mp1);
void            SetData( CAMRecPtr pcr, ULONG ulHndl, CAMObjectInfoPtr pObjInfo);
void            NextRec( HWND hCnr, CAMRecPtr * ppcrIn);
void            FreeLinkedRecords( HWND hCnr, CAMRecPtr pcr);
void            CalcCnrTotals( HWND hCnr);
void            AdjustList( CAMObjectHandlePtr pHandles);
void            CloseLoadingDlg( HWND hwnd);

/****************************************************************************/

static BOOL     fInitColMsgs        = FALSE;
static char     szX[]               = "xXx";

static char *   pszCOL_TooBig       = szX;
static char *   aszDay[]            = {szX, szX, szX, szX, szX, szX, szX};

static CAMNLSPSZ nlsColMsgs[] = {
    { &pszCOL_TooBig,       COL_TooBig },
    { &aszDay[0],           COL_Sun },
    { &aszDay[1],           COL_Mon },
    { &aszDay[2],           COL_Tue },
    { &aszDay[3],           COL_Wed },
    { &aszDay[4],           COL_Thu },
    { &aszDay[5],           COL_Fri },
    { &aszDay[6],           COL_Sat },
    { 0,                    0 }};

/****************************************************************************/

void        GetData( HWND hwnd, BOOL fSync)

{
    BOOL            fRtn = FALSE;
    HWND            hCnr;
    ULONG           ulErr = STS_GetList;
    char *          pErr = "";
    CAMGetData *    pcgd = 0;
    CAMRecPtr       pcr;
    CNRINFO         cnri;

do
{
    // show the "please wait" dialog
    OpenWindow( hwnd, IDD_LOADING);
    hCnr = WinWindowFromID( hwnd, FID_CLIENT);

    if (!fInitColMsgs) {
        LoadStaticStrings( nlsColMsgs);
        fInitColMsgs = TRUE;
    }

    // if there aren't any records, use GetList instead of SyncList;
    // otherwise, empty the container if GetList was requested
    pcr = 0;
    NextRec( hCnr, &pcr);
    if (!pcr)
        fSync = FALSE;
    else
        if (!fSync) {
            while (pcr) {
                if (pcr->bmp)
                    GpiDeleteBitmap( pcr->bmp);
                if (pcr->orgname) 
                    free( pcr->orgname);

                NextRec( hCnr, &pcr);
            }

            if (WinSendMsg( hCnr, CM_REMOVERECORD, 0,
                            MPFROM2SHORT( 0, CMA_FREE | CMA_INVALIDATE)) != 0)
                printf( "GetData - CM_REMOVERECORD failed\n");
        }

    // set the sort function to be used when inserting records
    cnri.cb = sizeof(CNRINFO);
    cnri.pSortRecord = QuerySortFunctionPtr();
    WinSendMsg( hCnr, CM_SETCNRINFO, (MP)&cnri, (MP)CMA_PSORTRECORD);

    if (CAM_DEMO_FLAG) {
        InitColumnWidths( hCnr);
        ulErr = STS_Copyright;
        pErr = "demo mode";
        break;
    }

    pcgd = (CAMGetData*)malloc( sizeof(CAMGetData));
    if (!pcgd) {
        pErr = "malloc";
        break;
    }
    memset( pcgd, 0, sizeof(CAMGetData));

    pcgd->fSync    = fSync;
    pcgd->hReply   = hwnd;
    pcgd->thisCam  = (CAMPtr)GetCamPtr();

    if (_beginthread( GetDataThreadProc, 0, 0xF000, pcgd) == -1) {
        pErr = "_beginthread";
        break;
    }

    fRtn = TRUE;

} while (0);

    if (!fRtn) {
        printf( "GetData - %s\n", pErr);
        UpdateStatus( hwnd, ulErr);
        CloseLoadingDlg( hwnd);
        if (pcgd)
            free( pcgd);
    }

    return;
}

/****************************************************************************/

void        GetDataThreadProc( void * camgetdataptr)

{
    CAMGetData *    pcgd;
    BOOL            fMutex = FALSE;

do {
    pcgd  = (CAMGetData*)camgetdataptr;

    pcgd->rc = DosRequestMutexSem( pcgd->thisCam->hmtxCam, CAM_MUTEXWAIT);
    if (pcgd->rc)
        break;
    fMutex = TRUE;

    pcgd->rc = GetObjectHandles( pcgd->thisCam->hCam, 0xffffffff,
                          0, 0, &pcgd->pHandles);
    if (!pcgd->rc)
        break;

    printf( "GetDataThreadProc - GetObjectHandles #1 - rc= 0x%x\n",
            (int)pcgd->rc);
    if (pcgd->rc == CAMERR_NULLCAMERAPTR)
        break;

    pcgd->rc = ClearStall( pcgd->thisCam->hCam);
    if (pcgd->rc == CAMERR_NULLCAMERAPTR)
        break;

    DosSleep( 500);
    pcgd->rc = GetObjectHandles( pcgd->thisCam->hCam, 0xffffffff,
                                 0, 0, &pcgd->pHandles);
    if (!pcgd->rc)
        break;

    printf( "GetDataThreadProc - GetObjectHandles #2 - rc= 0x%x\n",
            (int)pcgd->rc);
    if (pcgd->rc != CAMERR_NULLCAMERAPTR)
        ClearStall( pcgd->thisCam->hCam);

} while (0);

    if (fMutex)
        DosReleaseMutexSem( pcgd->thisCam->hmtxCam);

    if (!WinPostMsg( pcgd->hReply, CAMMSG_GETDATA, (MP)pcgd, 0))
        printf( "GetDataThreadProc - WinPostMsg failed\n");

    return;
}

/****************************************************************************/

void        GetDataReply( HWND hwnd, MPARAM mp1)

{
    CAMGetData *    pcgd = (CAMGetData*)mp1;
    BOOL            fRtn = FALSE;
    HWND            hCnr;
    ULONG           ulErr = STS_GetList;

do {
    hCnr = WinWindowFromID( hwnd, FID_CLIENT);

    if (pcgd->rc)
        break;

    // Dave's fix
    if (CAM_DAVE_FLAG)
        AdjustList( pcgd->pHandles);

    if (!pcgd->pHandles->cntHandles) {
        ulErr = STS_NoFiles;
        break;
    }

    if (pcgd->fSync)
        fRtn = SyncList( hCnr, pcgd);
    else
        fRtn = GetList( hCnr, pcgd);

} while (0);

    if (!fRtn) {
        UpdateStatus( hwnd, ulErr);
        CloseLoadingDlg( hwnd);

        if (pcgd->pHandles)
            free( pcgd->pHandles);
        if (pcgd->pcrFirst)
            FreeLinkedRecords( hCnr, pcgd->pcrFirst);
        if (pcgd->ppcrSync)
            free( pcgd->ppcrSync);
        free( pcgd);
    }

    return;
}

/****************************************************************************/

BOOL        GetList( HWND hCnr, CAMGetData * pcgd)

{
    BOOL            fRtn = FALSE;
    char *          pErr = "";

do
{
    pcgd->pcrFirst = (CAMRecPtr)WinSendMsg( hCnr, CM_ALLOCRECORD,
                                           (MP)CAMREC_EXTRABYTES,
                                           (MP)pcgd->pHandles->cntHandles);
    if (!pcgd->pcrFirst) {
        pErr = "CM_ALLOCRECORD";
        break;
    }

    if (_beginthread( GetListThreadProc, 0, 0xF000, pcgd) == -1) {
        pErr = "_beginthread";
        break;
    }

    fRtn = TRUE;

} while (0);

    if (!fRtn)
        printf( "GetList - %s\n", pErr);

    return (fRtn);
}

/****************************************************************************/

void        GetListThreadProc( void * camgetdataptr)

{
    CAMGetData *     pcgd;
    BOOL             fMutex = FALSE;
    PULONG           pHndl;
    ULONG            cnt;
    CAMRecPtr        pcr;
    CAMObjectInfoPtr pObjInfo = 0;

do {
    pcgd  = (CAMGetData*)camgetdataptr;
    pHndl = (ULONG*)pcgd->pHandles->Handles;
    cnt   = pcgd->pHandles->cntHandles;
    pcr   = pcgd->pcrFirst;

    pcgd->rc = DosRequestMutexSem( pcgd->thisCam->hmtxCam, CAM_MUTEXWAIT);
    if (pcgd->rc)
        break;
    fMutex = TRUE;

    // for each handle, get its info then format a record with its data
    for ( ; pcr && cnt; cnt--, pHndl++) {

        pcgd->rc = GetObjectInfo( pcgd->thisCam->hCam, *pHndl, &pObjInfo);
        if (pcgd->rc) {
            printf( "GetListThreadProc - GetObjectInfo #1 - rc= 0x%x\n", (int)pcgd->rc);
            if (pcgd->rc == CAMERR_NULLCAMERAPTR)
                break;

            pcgd->rc = ClearStall( pcgd->thisCam->hCam);
            if (pcgd->rc == CAMERR_NULLCAMERAPTR)
                break;

            DosSleep( 500);
            pcgd->rc = GetObjectInfo( pcgd->thisCam->hCam, *pHndl, &pObjInfo);
            if (pcgd->rc) {
                printf( "GetListThreadProc - GetObjectInfo #2 - rc= 0x%x\n", (int)pcgd->rc);
                if (pcgd->rc != CAMERR_NULLCAMERAPTR)
                    ClearStall( pcgd->thisCam->hCam);
                break;
            }
        }

        // construct a record for everything except folders
        if (pObjInfo->ObjectFormat != PTP_OFC_Association) {
            SetData( pcr, *pHndl, pObjInfo);
            pcgd->cntRecords++;
            pcgd->pcrLast = pcr;
            pcr = (CAMRecPtr)pcr->mrc.preccNextRecord;
        }

        free( pObjInfo);
        pObjInfo = 0;
    }

    if (pObjInfo)
        free( pObjInfo);

} while (0);

    if (fMutex)
        DosReleaseMutexSem( pcgd->thisCam->hmtxCam);

    if (!WinPostMsg( pcgd->hReply, CAMMSG_GETLIST, (MP)pcgd, 0))
        printf( "GetListThreadProc - WinPostMsg failed\n");

    return;
}

/****************************************************************************/

void        GetListReply( HWND hwnd, MPARAM mp1)

{
    CAMGetData *    pcgd = (CAMGetData*)mp1;
    HWND            hCnr;
    LONG            rc;
    ULONG           ulErr;
    CAMRecPtr       pcrDel = 0;
    RECORDINSERT    ri;

    hCnr = WinWindowFromID( hwnd, FID_CLIENT);

    if (pcgd->rc)
        rc = -1;
    else
    if (!pcgd->cntRecords)
        rc = 0;
    else {
        rc = 1;

        // break the linked list at pcrLast which was the last record used
        pcrDel = (CAMRecPtr)pcgd->pcrLast->mrc.preccNextRecord;
        pcgd->pcrLast->mrc.preccNextRecord = 0;

        // init the recordinsert struct
        ri.cb = sizeof(RECORDINSERT);
        ri.pRecordOrder = (PRECORDCORE)CMA_FIRST;
        ri.pRecordParent = 0;
        ri.fInvalidateRecord = TRUE;
        ri.zOrder = CMA_TOP;
        ri.cRecordsInsert = pcgd->cntRecords;

        // insert all the records at once;  if this fails, relink the list
        if (!WinSendMsg( hCnr, CM_INSERTRECORD, (MP)pcgd->pcrFirst, (MP)&ri)) {
            pcgd->pcrLast->mrc.preccNextRecord = (CAM_RECTYPE*)pcrDel;
            rc = -1;
        }
    }

    // if there were any errors, set up to delete every record
    if (rc != 1)
        pcrDel = pcgd->pcrFirst;

    // free any records that weren't used;  normally,
    // these reflect folder entries that weren't listed
    FreeLinkedRecords( hCnr, pcrDel);

    if (rc == -1)
        ulErr = STS_ListingFiles;
    else
    if (rc == 0)
        ulErr = STS_NoFiles;
    else {
        ulErr = STS_Copyright;
        InitColumnWidths( hCnr);
        FetchAllThumbnails( hwnd);
    }

    // get rid of the wait dialog, then update the status bar
    CalcCnrTotals( hCnr);
    UpdateStatus( hwnd, ulErr);
    CloseLoadingDlg( hwnd);

    free( pcgd->pHandles);
    free( pcgd);

    return;
}

/****************************************************************************/
/****************************************************************************/

BOOL        SyncList( HWND hCnr, CAMGetData * pcgd)

{
    BOOL            fRtn = FALSE;
    ULONG           ctr;
    CAMRecPtr *     ppcr;
    CAMRecPtr *     ppcrSav;
    char *          pErr = "";
    CNRINFO         ci;

do {
    if (!WinSendMsg( hCnr, CM_QUERYCNRINFO, (MP)&ci, (MP)sizeof(CNRINFO))) {
        pErr = "CM_QUERYCNRINFO";
        break;
    }

    ctr = ci.cRecords * sizeof(CAMRecPtr);
    pcgd->ppcrSync = (CAMRecPtr*)malloc( ctr);
    if (!pcgd->ppcrSync) {
        pErr = "malloc";
        break;
    }
    memset( pcgd->ppcrSync, 0, ctr);

    ppcr = pcgd->ppcrSync;
    *ppcr = (CAMRecPtr)WinSendMsg( hCnr, CM_QUERYRECORD, (MP)0,
                                   MPFROM2SHORT( CMA_FIRST, CMA_ITEMORDER));

    ctr = 0;
    while (*ppcr && *ppcr != (CAMRecPtr)-1 && ctr < ci.cRecords) {
        ppcrSav = ppcr;
        ctr++;
        ppcr++;
        *ppcr = (CAMRecPtr)WinSendMsg( hCnr, CM_QUERYRECORD, (MP)*ppcrSav,
                                       MPFROM2SHORT( CMA_NEXT, CMA_ITEMORDER));
    }

    if (*ppcr) {
        if (*ppcr == (CAMRecPtr)-1)
            pErr = "CM_QUERYRECORD";
        else
            pErr = "record overflow";
        break;
    }

    if (ctr < ci.cRecords)
        printf( "GetList - record count mismatch - cRecords= %d  ctr= %d\n",
                (int)ci.cRecords, (int)ctr);

    pcgd->cntSync  = ctr;
    pcgd->pcrFirst = (CAMRecPtr)WinSendMsg( hCnr, CM_ALLOCRECORD, (MP)CAMREC_EXTRABYTES,
                                    (MP)(pcgd->pHandles->cntHandles - ci.cRecords));
    if (!pcgd->pcrFirst) {
        pErr = "CM_ALLOCRECORD";
        break;
    }

    if (_beginthread( SyncListThreadProc, 0, 0xF000, pcgd) == -1) {
        pErr = "_beginthread";
        break;
    }

    fRtn = TRUE;

} while (0);

    if (!fRtn)
        printf( "SyncList - %s\n", pErr);

    return (fRtn);
}

/****************************************************************************/

void        SyncListThreadProc( void * camgetdataptr)

{
    CAMGetData *     pcgd;
    BOOL             fMutex = FALSE;
    PULONG           pHndl;
    ULONG            cntHndl;
    ULONG            ctr;
    ULONG            ndxSync;
    CAMRecPtr        pcr;
    CAMRecPtr *      ppcr;
    CAMObjectInfoPtr pObjInfo = 0;

do {
    pcgd    = (CAMGetData*)camgetdataptr;
    pHndl   = (ULONG*)pcgd->pHandles->Handles;
    cntHndl = pcgd->pHandles->cntHandles;
    pcr     = pcgd->pcrFirst;
    ppcr    = pcgd->ppcrSync;
    ndxSync = 0;

    pcgd->rc = DosRequestMutexSem( pcgd->thisCam->hmtxCam, CAM_MUTEXWAIT);
    if (pcgd->rc)
        break;
    fMutex = TRUE;

    // review each handle on the list
    for ( ; cntHndl; cntHndl--, pHndl++) {

        // freeing pObjInfo here makes it easier to use 'continue' below
        if (pObjInfo) {
            free( pObjInfo);
            pObjInfo = 0;
        }

        pcgd->rc = GetObjectInfo( pcgd->thisCam->hCam, *pHndl, &pObjInfo);
        if (pcgd->rc) {
            printf( "SyncListThreadProc - GetObjectInfo #1 - rc= 0x%x\n", (int)pcgd->rc);
            if (pcgd->rc == CAMERR_NULLCAMERAPTR)
                break;

            pcgd->rc = ClearStall( pcgd->thisCam->hCam);
            if (pcgd->rc == CAMERR_NULLCAMERAPTR)
                break;

            DosSleep( 500);
            pcgd->rc = GetObjectInfo( pcgd->thisCam->hCam, *pHndl, &pObjInfo);
            if (pcgd->rc) {
                printf( "SyncListThreadProc - GetObjectInfo #2 - rc= 0x%x\n", (int)pcgd->rc);
                if (pcgd->rc != CAMERR_NULLCAMERAPTR)
                    ClearStall( pcgd->thisCam->hCam);
                break;
            }
        }

        // if this is a folder, skip it
        if (pObjInfo->ObjectFormat == PTP_OFC_Association)
            continue;

        // see if there's an existing record that matches this handle;
        // if so, zero-out the pointer to this record so that when we're
        // done, only records that weren't matched will remain;
        // ndxSync is a minor optimization to avoid going over the
        // beginning of the list repeatedly
        for (ctr = ndxSync; ctr < pcgd->cntSync; ctr++) {
            if (ppcr[ctr] == 0)
                continue;

            if (!strcmp( pObjInfo->Filename, (ppcr[ctr]->orgname ?
                         ppcr[ctr]->orgname : ppcr[ctr]->title))) {
                ppcr[ctr]->hndl = *pHndl;
                if (ctr == ndxSync)
                    ndxSync++;
                ppcr[ctr] = 0;
                break;
            }
        }

        // if a match was found, continue
        if (ctr < pcgd->cntSync)
            continue;

        // otherwise, format a new record
        SetData( pcr, *pHndl, pObjInfo);
        pcgd->cntRecords++;
        pcgd->pcrLast = pcr;
        pcr = (CAMRecPtr)pcr->mrc.preccNextRecord;
    }

    if (pObjInfo)
        free( pObjInfo);

} while (0);

    if (fMutex)
        DosReleaseMutexSem( pcgd->thisCam->hmtxCam);

    if (!WinPostMsg( pcgd->hReply, CAMMSG_SYNCLIST, (MP)pcgd, 0))
        printf( "SyncListThreadProc - WinPostMsg failed\n");

    return;
}

/****************************************************************************/

void        GetSyncReply( HWND hwnd, MPARAM mp1)

{
    CAMGetData *    pcgd = (CAMGetData*)mp1;
    HWND            hCnr;
    BOOL            fRtn;
    ULONG           ctr;
    ULONG           ulErr;
    CAMRecPtr       pcrDel;
    CAMRecPtr *     ppcr;
    RECORDINSERT    ri;

    hCnr = WinWindowFromID( hwnd, FID_CLIENT);
    fRtn = (pcgd->rc ? FALSE : TRUE);

    if (!fRtn || !pcgd->cntRecords)
        pcrDel = pcgd->pcrFirst;
    else {
        // break the linked list at pcrLast which was the last record used
        pcrDel = (CAMRecPtr)pcgd->pcrLast->mrc.preccNextRecord;
        pcgd->pcrLast->mrc.preccNextRecord = 0;

        // init the recordinsert struct
        ri.cb = sizeof(RECORDINSERT);
        ri.pRecordOrder = (PRECORDCORE)CMA_FIRST;
        ri.pRecordParent = 0;
        ri.fInvalidateRecord = TRUE;
        ri.zOrder = CMA_TOP;
        ri.cRecordsInsert = pcgd->cntRecords;

        // insert all the records at once;  if this fails, relink the list
        if (!WinSendMsg( hCnr, CM_INSERTRECORD, (MP)pcgd->pcrFirst, (MP)&ri)) {
            pcgd->pcrLast->mrc.preccNextRecord = (CAM_RECTYPE*)pcrDel;
            pcrDel = pcgd->pcrFirst;
            fRtn = FALSE;
        }
    }

    // free any new records that weren't used
    FreeLinkedRecords( hCnr, pcrDel);

    // free existing records that weren't matched
    if (fRtn)
        for (ppcr = pcgd->ppcrSync, ctr = 0; ctr < pcgd->cntSync; ctr++) {
            if (ppcr[ctr] == 0)
                continue;

            if (ppcr[ctr]->bmp)
                GpiDeleteBitmap( ppcr[ctr]->bmp);
            if (ppcr[ctr]->orgname)
                free( ppcr[ctr]->orgname);
            WinSendMsg( hCnr, CM_REMOVERECORD, (MP)&ppcr[ctr],
                        MPFROM2SHORT( 1, CMA_FREE));
        }

    if (!fRtn)
        ulErr = STS_RefreshList;
    else {
        ulErr = STS_Copyright;
        FetchAllThumbnails( hwnd);
    }

    // get rid of the wait dialog, then update the status bar
    CalcCnrTotals( hCnr);
    UpdateStatus( hwnd, ulErr);
    CloseLoadingDlg( hwnd);

    free( pcgd->pHandles);
    free( pcgd->ppcrSync);
    free( pcgd);

    return;
}

/****************************************************************************/
/****************************************************************************/

void        SetData( CAMRecPtr pcr, ULONG ulHndl, CAMObjectInfoPtr pObjInfo)

{
    struct tm *     ptm;
    char            szText[32];

    // mark & status
    pcr->mark = GetStatStuff()->hBlkIco;
    pcr->status = 0;

    if (pObjInfo->ProtectionStatus)
        pcr->status |= STAT_PROTECTED;
    if (pObjInfo->ProtectionStatus > 1)
        pcr->status |= STAT_PROTECTED2;

    // number & handle
    pcr->nbr = 0;
    pcr->hndl = ulHndl;

    // title
    pcr->title = pcr->sztitle;
    strcpy( pcr->sztitle, pObjInfo->Filename);
    pcr->orgname = 0;

    // group
    pcr->grpptr = GetDefaultGroup( &pcr->group);

    // image format
    pcr->fmt = GetFormatName( pObjInfo->ObjectFormat);
    pcr->fmtnbr = pObjInfo->ObjectFormat;

    // image XY
    pcr->picx = pObjInfo->ImagePixWidth;
    pcr->picy = pObjInfo->ImagePixHeight;
    if (sprintf( szText, "%dx%d",
        pObjInfo->ImagePixWidth, pObjInfo->ImagePixHeight) > 11)
        strcpy( szText, pszCOL_TooBig);
    pcr->picxy = pcr->szpicxy;
    strcpy( pcr->szpicxy, szText);

    // thumb format
    pcr->tnfmt = GetFormatName( pObjInfo->ThumbFormat);
    pcr->tnfmtnbr = pObjInfo->ThumbFormat;

    // thumb XY
    pcr->tnx = pObjInfo->ThumbPixWidth;
    pcr->tny = pObjInfo->ThumbPixHeight;
    if (sprintf( szText, "%dx%d",
        pObjInfo->ThumbPixWidth, pObjInfo->ThumbPixHeight) > 11)
        strcpy( szText, pszCOL_TooBig);
    pcr->tnxy = pcr->sztnxy;
    strcpy( pcr->sztnxy, szText);

    // thumb size
    pcr->tnsize = pObjInfo->ThumbCompressedSize;

    // image size
    pcr->size = pObjInfo->ObjectCompressedSize;

    ptm = localtime( &pObjInfo->CaptureDate);

    // day
    pcr->day = aszDay[ptm->tm_wday];

    // date
    pcr->date.day   = (UCHAR)ptm->tm_mday;
    pcr->date.month = (UCHAR)ptm->tm_mon+1;
    pcr->date.year  = (USHORT)ptm->tm_year+1900;

    // time
    pcr->time.hours   = ptm->tm_hour;
    pcr->time.minutes = ptm->tm_min;
    pcr->time.seconds = ptm->tm_sec;

    // icon view info - the pszIcon field of the (MINI)RECORDCORE
    // has to have something or the container won't allocate space
    // for a title when in icon view;  the icon or bitmap field can
    // be empty because the container is ownerdraw in icon view
    pcr->mrc.pszIcon = pcr->sztitle;
    pcr->bmp = 0;

    return;
}

/****************************************************************************/

void        NextRec( HWND hCnr, CAMRecPtr * ppcrIn)

{
    *ppcrIn = (CAMRecPtr)WinSendMsg( hCnr, CM_QUERYRECORD, (MP)*ppcrIn,
               MPFROM2SHORT( (*ppcrIn ? CMA_NEXT : CMA_FIRST), CMA_ITEMORDER));

    if (*ppcrIn == (CAMRecPtr)-1)
        *ppcrIn = 0;

    return;
}

/****************************************************************************/

void        FreeLinkedRecords( HWND hCnr, CAMRecPtr pcr)

{
    CAMRecPtr   pcrSav;

    while (pcr) {
        pcrSav = (CAMRecPtr)pcr->mrc.preccNextRecord;
        if (!WinSendMsg( hCnr, CM_FREERECORD, (MP)&pcr, (MP)1))
            printf( "FreeLinkedRecords -  CM_FREERECORD\n");
        pcr = pcrSav;
    }

    return;
}

/****************************************************************************/

void        CalcCnrTotals( HWND hCnr)

{
    CAMRecPtr   pcr;
    STATSTUFF * pss;

    // init the counters
    pss = GetStatStuff();
    pss->ulTotCnt = 0;
    pss->ulTotSize = 0;
    pss->ulSavCnt = 0;
    pss->ulSavSize = 0;
    pss->ulDelCnt = 0;
    pss->ulDelSize = 0;

    pcr = 0;
    NextRec( hCnr, &pcr);

    while (pcr) {
        pss->ulTotSize += pcr->size;
        pss->ulTotCnt++;
        pcr->nbr = pss->ulTotCnt;

        if (STAT_NOTSAV(pcr->status)) {
            pss->ulSavCnt++;
            pss->ulSavSize += pcr->size;
        }
        else
        if (STAT_ISDEL(pcr->status)) {
            pss->ulDelCnt++;
            pss->ulDelSize += pcr->size;
        }
        NextRec( hCnr, &pcr);
    }

    return;
}

/****************************************************************************/

// The list of file handles generated by Dave M's camera had invalid entries
// at the beginning, followed by valid ones.  The invalid ones could be
// identified because their numbers were out of sequence.  This works around
// the issue by starting at the end of the list and ensuring that each
// preceeding entry has a lower handle number.  When it finds an out-of-
// sequence entry, it assumes that everything from that point back to the
// beginning is invalid and adjusts the list pointer & count accordingly.

void        AdjustList( CAMObjectHandlePtr pHandles)

{
    ULONG               ctr;
    ULONG               last;
    PULONG              pHndl;

    ctr = pHandles->cntHandles;
    pHndl = (ULONG*)pHandles->Handles;
    last = ~0;

    while (ctr) {
        ctr--;
        if (pHndl[ctr] >= last) {
            ctr++;
            break;
        }
        last = pHndl[ctr];
    }

    if (ctr) {
        printf( "AdjustList - ignoring first %d of %d files\n",
                (int)ctr, (int)pHandles->cntHandles);
        pHandles->Handles = (uint32_t*)&pHndl[ctr];
        pHandles->cntHandles -= ctr;
    }

    return;
}

/****************************************************************************/

void        CloseLoadingDlg( HWND hwnd)

{
    HWND    hDlg = GetHwnd( IDD_LOADING);

    if (hDlg) {
        WindowClosed( IDD_LOADING);
        WinDestroyWindow( hDlg);
    }

    return;
}

/****************************************************************************/

